@@ -157,7 +157,7 @@ def run_tests():
 
 
 def obtain_arguments(soname, standard="C++11", optimisations=["-march=native", "-O3"],
-                    options=["-Wall", "-Wextra", "-fdiagnostics-color=always"], **unused_kwargs):
+                    options=["-g", "-Wall", "-Wextra", "-fdiagnostics-color=always"], **unused_kwargs):
 
     so_args = ["-fPIC", "-shared", "-o", soname, "lifelib.cpp"]
 
