@@ -11,7 +11,7 @@ namespace apg {
         1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
         1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1};
 
-    void bits2bytes_sse2(uint64_t *inleafx, uint64_t *quarter) {
+    void bits2bytes_sse2(const uint64_t *inleafx, uint64_t *quarter) {
 
         asm (
         // Load data from memory into registers:
@@ -92,7 +92,7 @@ namespace apg {
 
     }
 
-    void bits2bytes(uint64_t *inleafx, uint64_t *quarter) {
+    void bits2bytes(const uint64_t *inleafx, uint64_t *quarter) {
         // Unpacks an inleaf into a 16-by-16 byte array, each element of
         // which is either 0 or 1:
         for (uint64_t x = 0; x < 2; x++) {
@@ -105,7 +105,7 @@ namespace apg {
         }
     }
 
-    void bytes2bits(uint64_t *quarter, uint64_t *outleafx) {
+    void bytes2bits(const uint64_t *quarter, uint64_t *outleafx) {
         for (uint64_t x = 0; x < 2; x++) {
             for (uint64_t y = 0; y < 2; y++) {
                 uint64_t l = 0;
@@ -117,7 +117,7 @@ namespace apg {
         }
     }
 
-    void p6dwordcycle_sse2(uint64_t *a, uint64_t *b) {
+    void p6dwordcycle_sse2(const uint64_t *a, uint64_t *b) {
 
         asm (
         "movups (%0), %%xmm0 \n\t"
@@ -146,7 +146,7 @@ namespace apg {
         
     }
 
-    void p6bytecycle_sse2(uint64_t *a, uint64_t *b) {
+    void p6bytecycle_sse2(const uint64_t *a, uint64_t *b) {
 
         asm (
         "movups (%0), %%xmm0 \n\t"
@@ -187,7 +187,7 @@ namespace apg {
 
     }
 
-    void p6dwordcycle_avx(uint64_t *a, uint64_t *b) {
+    void p6dwordcycle_avx(const uint64_t *a, uint64_t *b) {
 
         asm (
         "vmovups (%0), %%xmm0 \n\t"
@@ -212,7 +212,7 @@ namespace apg {
         
     }
 
-    void p6bytecycle_avx(uint64_t *a, uint64_t *b) {
+    void p6bytecycle_avx(const uint64_t *a, uint64_t *b) {
 
         asm (
         "vmovups (%0), %%xmm0 \n\t"
@@ -280,7 +280,7 @@ namespace apg {
         : "xmm0", "xmm1", "memory" );
     }
 
-    void parallel_diff_sse2(uint64_t *a, uint64_t *b, uint64_t *c) {
+    void parallel_diff_sse2(const uint64_t *a, const uint64_t *b, uint64_t *c) {
         asm (
 #include "bytediff_sse2.asm"
         : /* no output operands -- implicitly volatile */
@@ -296,7 +296,7 @@ namespace apg {
         : "xmm0", "xmm1", "memory" );
     }
 
-    void parallel_diff_avx(uint64_t *a, uint64_t *b, uint64_t *c) {
+    void parallel_diff_avx(const uint64_t *a, const uint64_t *b, uint64_t *c) {
         asm (
 #include "bytediff_avx.asm"
         : /* no output operands -- implicitly volatile */
@@ -304,7 +304,7 @@ namespace apg {
         : "xmm0", "xmm1", "memory" );
     }
 
-    void hadd_avx(uint64_t *a, uint64_t *b, uint64_t *c, uint64_t *d) {
+    void hadd_avx(const uint64_t *a, const uint64_t *b, const uint64_t *c, uint64_t *d) {
         asm (
 #include "hadd_avx.asm"
         : /* no output operands -- implicitly volatile */
@@ -312,7 +312,7 @@ namespace apg {
         : "xmm0", "xmm1", "memory" );
     }
 
-    void hadd_avx2(uint64_t *a, uint64_t *b, uint64_t *c, uint64_t *d) {
+    void hadd_avx2(const uint64_t *a, const uint64_t *b, const uint64_t *c, uint64_t *d) {
         asm (
 #include "hadd_avx2.asm"
         : /* no output operands -- implicitly volatile */
@@ -320,7 +320,7 @@ namespace apg {
         : "xmm0", "xmm1", "memory" );
     }
 
-    void hadd_sse2(uint64_t *a, uint64_t *b, uint64_t *c, uint64_t *d) {
+    void hadd_sse2(const uint64_t *a, const uint64_t *b, const uint64_t *c, uint64_t *d) {
         asm (
 #include "hadd_sse2.asm"
         : /* no output operands -- implicitly volatile */
@@ -328,7 +328,7 @@ namespace apg {
         : "xmm0", "xmm1", "memory" );
     }
 
-    void ladd_avx(uint64_t *a, uint64_t *b, uint64_t *c, uint64_t *d) {
+    void ladd_avx(const uint64_t *a, const uint64_t *b, const uint64_t *c, uint64_t *d) {
         asm (
 #include "ladd_avx.asm"
         : /* no output operands -- implicitly volatile */
@@ -336,7 +336,7 @@ namespace apg {
         : "xmm0", "xmm1", "memory" );
     }
 
-    void ladd_avx2(uint64_t *a, uint64_t *b, uint64_t *c, uint64_t *d) {
+    void ladd_avx2(const uint64_t *a, const uint64_t *b, const uint64_t *c, uint64_t *d) {
         asm (
 #include "ladd_avx2.asm"
         : /* no output operands -- implicitly volatile */
@@ -344,7 +344,7 @@ namespace apg {
         : "xmm0", "xmm1", "memory" );
     }
 
-    void ladd_sse2(uint64_t *a, uint64_t *b, uint64_t *c, uint64_t *d) {
+    void ladd_sse2(const uint64_t *a, const uint64_t *b, const uint64_t *c, uint64_t *d) {
         asm (
 #include "ladd_sse2.asm"
         : /* no output operands -- implicitly volatile */
@@ -406,7 +406,7 @@ namespace apg {
         destructive_convolve(c, b, range);
     }
 
-    void cmpxor_sse2(uint64_t *a, uint64_t *b, uint64_t *c) {
+    void cmpxor_sse2(const uint64_t *a, const uint64_t *b, uint64_t *c) {
         asm (
         "movups (%0), %%xmm0 \n\t"
         "movups (%1), %%xmm1 \n\t"
@@ -441,7 +441,7 @@ namespace apg {
         : "xmm0", "xmm1", "xmm2", "memory" );
     }
 
-    void hrot_ssse3(uint64_t *cc, uint64_t *nc, const uint8_t *ruledata) {
+    void hrot_ssse3(const uint64_t *cc, uint64_t *nc, const uint8_t *ruledata) {
         asm (
         "pxor %%xmm2, %%xmm2 \n\t"
         "movups 32(%2), %%xmm5 \n\t"
