@@ -251,6 +251,106 @@ class iwriter_base(object):
         self.f.write('    }\n\n')
 
 
+def rule2gates(rulestring):
+
+    bee = [0] * 10
+    ess = [0] * 10
+
+    for char in rulestring:
+        if (char == 'b'):
+            birth = True
+        elif (char == 's'):
+            birth = False
+        else:
+            k = int(char)
+            if (birth):
+                bee[k] = 1
+            else:
+                ess[k+1] = 1
+
+    negate = bee[0]
+    beexor = (bee[0] != bee[8])
+    essxor = (ess[1] != ess[9])
+
+    stars = ("*" if essxor else "") + ("**" if beexor else "") + ("!" if negate else "")
+
+    if negate:
+        bee = [1 - x for x in bee]
+        ess = [1 - x for x in ess]
+
+    ess[0] = (1 - ess[8]) if essxor else ess[8]
+
+    ruleint = 0;
+    for i in range(8):
+        ruleint += (bee[i] << i)
+        ruleint += (ess[i] << (i + 8))
+
+    rident = None
+
+    with open('../boolean.out', 'r') as f:
+        for fline in f:
+            x = fline.split(":  ")
+            if (len(x) == 2) and (int(x[0]) == ruleint):
+                rident = x[1][:-1]
+
+    if (ruleint == 0):
+        rident = '-004'
+    if (ruleint == 65280):
+        rident = '-331'
+    if (ruleint == 61680):
+        rident = '-221'
+    if (ruleint == 52428):
+        rident = '-111'
+    if (ruleint == 43690):
+        rident = '-001'
+
+    if rident is None:
+        raise ValueError("Error: unrecognised rule")
+    # print(("Rule circuit:     ["+rident+"]"+stars))
+
+    rchars = list(rident)
+    for i in range(len(rchars)):
+        if (rchars[i] == '-'):
+            rchars[i] = (i//4) + 4
+        else:
+            rchars[i] = int(rchars[i])
+
+    for i in range(0, len(rchars), 4):
+        if (rchars[i+3] == 3):
+            rchars[i+3] = 2
+            rchars[i+2] ^= rchars[i+1]
+            rchars[i+1] ^= rchars[i+2]
+            rchars[i+2] ^= rchars[i+1]
+
+    # Map logical registers to physical registers:
+    for i in range(0, len(rchars), 4):
+        dependencies = [False] * 10
+        for j in range(i+4, len(rchars), 4):
+            dependencies[rchars[j+1]] = True
+            dependencies[rchars[j+2]] = True
+        d = -1
+        e = rchars[i]
+        for j in range(6):
+            if (e == j):
+                d = j
+                break
+            if not dependencies[j]:
+                d = j
+                break
+        if (d == -1):
+            raise RuntimeError("Error: insufficiently many physical registers")
+        else:
+            # print(str(e)+" --> "+str(d))
+            rchars[i] = d
+            for j in range(i+4, len(rchars), 4):
+                if (rchars[j+1] == e):
+                    rchars[j+1] = d
+                if (rchars[j+2] == e):
+                    rchars[j+2] = d
+
+    return (rchars, negate, beexor, essxor)
+
+
 class iwriter(iwriter_base):
 
     def load_and_hshift(self, i, oddgen, terminal):
@@ -325,32 +425,8 @@ class iwriter(iwriter_base):
 
     def genlogic(self, rulestring):
 
-        bee = [0] * 10
-        ess = [0] * 10
+        rchars, negate, beexor, essxor = rule2gates(rulestring)
 
-        for char in rulestring:
-            if (char == 'b'):
-                birth = True
-            elif (char == 's'):
-                birth = False
-            else:
-                k = int(char)
-                if (birth):
-                    bee[k] = 1
-                else:
-                    ess[k+1] = 1
-
-        negate = bee[0]
-        beexor = (bee[0] != bee[8])
-        essxor = (ess[1] != ess[9])
-
-        stars = ("*" if essxor else "") + ("**" if beexor else "") + ("!" if negate else "")
-
-        if negate:
-            bee = [1 - x for x in bee]
-            ess = [1 - x for x in ess]
-
-        ess[0] = (1 - ess[8]) if essxor else ess[8]
         usetopbit = (essxor or beexor)
 
         self.logicgate('pand', 8, 11, 1)
@@ -364,87 +440,11 @@ class iwriter(iwriter_base):
             self.logicgate('pand', 12, 11, 11)
         self.logicgate('pxor', 1, 9, 9)
 
-        ruleint = 0;
-        for i in range(8):
-            ruleint += (bee[i] << i)
-            ruleint += (ess[i] << (i + 8))
-
-        # print(("Rule integer:     "+str(ruleint)+stars))
         regnames = [10, 8, 9, 12, 1, 0]
         opnames = ["and", "or", "andn", "nonsense", "xor"]
-        rident = None
-
-        with open('../boolean.out', 'r') as f:
-            for fline in f:
-                x = fline.split(":  ")
-                if (len(x) == 2) and (int(x[0]) == ruleint):
-                    rident = x[1][:-1]
-
-        if (ruleint == 0):
-            rident = '-004'
-        if (ruleint == 65280):
-            rident = '-331'
-        if (ruleint == 61680):
-            rident = '-221'
-        if (ruleint == 52428):
-            rident = '-111'
-        if (ruleint == 43690):
-            rident = '-001'
-
-        if rident is None:
-            raise ValueError("Error: unrecognised rule")
-        # print(("Rule circuit:     ["+rident+"]"+stars))
-
-        rchars = list(rident)
-        for i in range(len(rchars)):
-            if (rchars[i] == '-'):
-                rchars[i] = (i//4) + 4
-            else:
-                rchars[i] = int(rchars[i])
 
         for i in range(0, len(rchars), 4):
-            if (rchars[i+3] == 3):
-                rchars[i+3] = 2
-                rchars[i+2] ^= rchars[i+1]
-                rchars[i+1] ^= rchars[i+2]
-                rchars[i+2] ^= rchars[i+1]
-
-        # Map logical registers to physical registers:
-        for i in range(0, len(rchars), 4):
-            dependencies = [False] * 10
-            for j in range(i+4, len(rchars), 4):
-                dependencies[rchars[j+1]] = True
-                dependencies[rchars[j+2]] = True
-            d = -1
-            e = rchars[i]
-            for j in range(6):
-                if (e == j):
-                    d = j
-                    break
-                if not dependencies[j]:
-                    d = j
-                    break
-            if (d == -1):
-                raise RuntimeError("Error: insufficiently many physical registers")
-            else:
-                # print(str(e)+" --> "+str(d))
-                rchars[i] = d
-                for j in range(i+4, len(rchars), 4):
-                    if (rchars[j+1] == e):
-                        rchars[j+1] = d
-                    if (rchars[j+2] == e):
-                        rchars[j+2] = d
-
-        if (rulestring == 'b3s23'):
-            # Possibly more optimal due to instruction order:
-            self.logicgate('pxor', 9, 8, 8)
-            self.logicgate('por', 10, 12, 12)
-            self.logicgate('pxor', 9, 10, 10)
-            self.logicgate('pand', 12, 8, 8)
-            self.logicgate('pand', 8, 10, 10)
-        else:
-            for i in range(0, len(rchars), 4):
-                self.logicgate('p'+opnames[rchars[i+3]], regnames[rchars[i+1]], regnames[rchars[i+2]], regnames[rchars[i]])
+            self.logicgate('p'+opnames[rchars[i+3]], regnames[rchars[i+1]], regnames[rchars[i+2]], regnames[rchars[i]])
 
         if (usetopbit):
             # printcomment(g, 'correct for B8/S8 nonsense:')
