@@ -13,7 +13,7 @@
 #include <iostream>
 #include <unordered_map>
 
-#include "../../../cpads/include/cpads/indextable.hpp"
+#include "../cpads/include/cpads/indextable.hpp"
 
 
 namespace apg {
