@@ -9,7 +9,7 @@ namespace apg {
 
     void try_copy(std::string &s, char* buffer, int buflen) {
 
-        if (s.length() >= buflen) {
+        if ((buflen < 0) || (s.length() >= ((uint64_t) buflen))) {
             std::ostringstream ss;
             ss << "!" << (((s.length() >> 8) + 1) << 8);
             s = ss.str();
