@@ -14,7 +14,7 @@ fi
 
 cd ..
 
-pip3 install --user git+git://github.com/supposedly/nutshell.git
+pip3 install --user git+https://github.com/supposedly/nutshell.git
 pip3 install --user --upgrade tqdm
 
 set -e
